/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   Breakpoint manipulation
 *   this file is a part of xtrace project
 *
 * $Id: breakpoint.c,v 1.2 2002/09/23 14:50:02 drozd Exp $
 */

#include <stdio.h>

#include "xtrace.h"

LL_HEAD(__bptlist);

/* set the breakpoint on address */
int
xtr_set_bpt(struct opt *op, vaddr_t va)
{
	ptrdiff_t diff;
	struct symbol *symp;
	struct llhead *lp;
	struct bpt *bp;
	int ret = 0;

	LL_FOREACH(lp, &__bptlist) {

		bp = LL_ENTRY(lp, struct bpt, list);
		
		if (bp->va == va) {
			symp = xtr_find_symbol(va, &diff);
			dprintf((stderr, "*** bpt already set for %s(+%x)\n",
			    symp->name, diff));
			--ret;
			break;
		}
	}

	/* not set, create new */
	if (ret == 0) {
		bp = calloc(1, sizeof(struct bpt));
		if (bp == NULL) {
			perror("*** xtr_set_bpt: calloc()");
			exit(1);
		}

		bp->va = va;
		bp->codelen = sizeof(BPT);
		LL_INIT(&bp->list);
		(void) xtr_read_mem(op, bp->va, &bp->oldcode, bp->codelen);
		(void) xtr_write_mem(op, bp->va, BPT, bp->codelen);

		/* link it */
		ll_add(&bp->list, &__bptlist);
	}

	return (ret);
}

/*
 * Clear breakpoint
 */
int
xtr_clear_bpt(struct opt *op, vaddr_t va)
{
	ptrdiff_t diff;
	struct symbol *symp;
	struct llhead *lp;
	struct bpt *bp;
	int found = 0;

	LL_FOREACH(lp, &__bptlist) {
		bp = LL_ENTRY(lp, struct bpt, list);
		if (bp->va == va) {
			symp = xtr_find_symbol(va, &diff);
			dprintf((stderr, "*** clearing bpt for %x %s(+%x)\n",
			    va, symp->name, diff));
			++found;
			break;
		}
	}

	/* if found, clear it */
	if (found) {
		(void) xtr_write_mem(op, bp->va, &bp->oldcode, bp->codelen);
		ll_del(lp);
		free(bp);
	} else {
		symp = xtr_find_symbol(va, &diff);
		dprintf((stderr, "*** NO bpt set for %x %s(+%x)\n",
		    va, symp->name, diff));
	}

	return (found ? 0 : -1);
}

/*
 * Clear all breakpoints set
 */
void
xtr_clrbpts(struct opt *op)
{
	struct llhead *lp, *tmp;
	struct bpt *bp;
	int count = 0;

	dprintf((stderr, "*** xtr_clrbpts: deleting all breakpoints\n"));

	LL_FOREACH_SAFE(lp, tmp, &__bptlist) {
		
		bp = LL_ENTRY(lp, struct bpt, list);
		dprintf((stderr, "*** xtr_clrbpts: %x\n", bp->va));
		(void) xtr_write_mem(op, bp->va, &bp->oldcode, bp->codelen);
		ll_del(lp);
		free(bp);
		++count;
	}
	
	dprintf((stderr, "%d deleted\n", count));
}
