/*
 * Copyright (c) 2002 Silver Software Ltd
 * All rights reserved
 *
 * Author: Sergey Lyubka
 * Description:
 *   Linked list helper library
 *
 * $Id: llist.c,v 1.1 2002/09/13 14:29:12 drozd Exp $
 */

#include "llist.h"

static void add(struct llhead *, struct llhead *, struct llhead *);

/*
 * Internal function. Inserts a new entry into known location
 * between two entries. prev/next must be defined
 */
static void
add(struct llhead *new, struct llhead *prev, struct llhead *next)
{
	next->prev = new;
	new->next = next;
	new->prev = prev;
	prev->next = new;
}

/*
 * Add a new entry after the list head. Implements stack behaviour
 */
void
ll_add(struct llhead *new, struct llhead *head)
{
	add(new, head, head->next);
}

/*
 * Add a new entry before the list head. Implements queue behaviour
 */
void
ll_add_tail(struct llhead *new, struct llhead *head)
{
	add(new, head->prev, head);
}

/*
 * Delete an entry
 */
void
ll_del(struct llhead *entry)
{
	entry->next->prev = entry->prev;
	entry->prev->next = entry->next;
}
