/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   i386 - specific defines
 *   this file is a part of xtrace project
 *
 * $Id: llist.h,v 1.1 2002/09/13 14:29:12 drozd Exp $
 */

#ifndef __LLIST__H
#define __LLIST__H

struct llhead {
	struct llhead *prev, *next;
};

/*
 *  LL_INIT:  initializes the list
 *  LL_ENTRY: convert list pointer to the pointer to struct that
 *  	contains that list. t - type of a struct, p - ptr to list head,
 *  	n - name of a list node in a struct
 *  LL_FOREACH: loop over all elements in a list. h - head, p - iterator
 *  LL_FOREACH_SAFE: loop over all elements in a list. h - head,
 *  	p - iterator, tmp - temporary storage
 *  	this macro is safe to use for the list element removal.
 *  LL_EMPTRY: nonzero if the list is empty, 0 otherwise
 */

#define	LL_INIT(p)	((p)->next = (p)->prev = p)
#define LL_HEAD(n)	struct llhead n = { &n, &n }
#define	LL_ENTRY(p,t,n)	(t *)((char *)(p) -(unsigned long)(&((t *) 0)->n))
#define	LL_EMPTY(p)	((p)->next == (p))
#define	LL_FOREACH(p,h)	for (p = (h)->next; p != (h); p = p->next)
#define LL_FOREACH_SAFE(p,tmp,h)					\
	for (p = (h)->next, tmp = p->next; p != (h); p = tmp, tmp = p->next)

void ll_add(struct llhead *new, struct llhead *head);
void ll_add_tail(struct llhead *new, struct llhead *head);
void ll_del(struct llhead *entry);

#endif /* __LLIST__H */
