/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   shared library - related routines
 *   this file is a part of xtrace project
 *
 * $Id: sharedlibs.c,v 1.6 2002/09/25 09:17:23 drozd Exp $
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include "config.h"
#include "xtrace.h"
#include "elf.h"

/*
 * get list of dependant ELF binaries
 */
struct sharedlib **
xtr_get_deps(pid_t pid)
{
	static struct sharedlib libs[128], *plibs[128];
	char buf[1024], fname[1024], progname[1024];
	FILE *fp;
	int nbytes, i, fd, isstatic;
	Elf32_Ehdr *ep;
	vaddr_t base;
	
	/* read the executable full path */
#ifdef FREEBSD
	(void) snprintf(fname, sizeof(fname), "/proc/%d/file", pid);
#elif LINUX
	(void) snprintf(fname, sizeof(fname), "/proc/%d/exe", pid);
#endif
	nbytes = readlink(fname, progname, sizeof(progname));
	progname[nbytes] = '\0';

	libs[0].path = strdup(progname);
	libs[0].base = 0U;

	/* is it static ? */
	if ((fd = open(libs[0].path, O_RDONLY)) < 0) {
		dprintf((stderr, "*** xtr_get_deps: open(%s)", libs[0].path));
		free(libs[0].path);
		/* FIXME: UNLOCK and exit() should be here */
		_exit(1);
	}

	if ((ep = elf_init(fd)) == 0) {
		dprintf((stderr, "xtr_get_deps: elf_init() failed\n"));
		(void) close(fd);
		/* FIXME: UNLOCK and exit() should be here */
		_exit(1);
	}

	isstatic = elf_isstatic(ep);


	if (isstatic) {
		i = 1;
	} else {


		/*
		 * set tracing flags for dynamic linker
		 * FIXME:  UGLY !!! rewrite this piece of shit
		 */
		(void) setenv("LD_TRACE_LOADED_OBJECTS", "1", 1);
#ifdef FREEBSD
		(void) setenv("LD_TRACE_LOADED_OBJECTS_FMT1", "%p %x\n", 1);
		(void) unsetenv("LD_TRACE_LOADED_OBJECTS_FMT2");
#endif /* FREEBSD */

		if ((fp = popen(progname, "r")) == NULL)
			return (NULL);

		for (i = 1; fgets(buf, sizeof(buf), fp) != NULL; ) {
			
#ifdef FREEBSD
			if (sscanf(buf, "%s %x", fname, &base) == 2)
#elif LINUX
			if (sscanf(buf, "%*s %*s %s (%x)", fname, &base) == 2)
#else
#error "unsupported platform"
#endif /* FREEBSD */
			{
				libs[i].path = strdup(fname);
				libs[i].base = base;
				i++;
			}
		}
	}

	/*
	 * add dynamic linker to a list of dependancies
	 */
#ifdef FREEBSD
	if (!isstatic) {
		elf_dynloader(ep, &libs[i].path, &libs[i].base);
		i++;
	}
#endif /* FREEBSD */
	
	elf_fini(ep);

	/* null-terminate the array */
	plibs[i--] = NULL;
	
	for (; i >= 0; i--)
		plibs[i] = &libs[i];

	/* restore environment */
	(void) unsetenv("LD_TRACE_LOADED_OBJECTS");

	return (plibs);
}
