/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   System V RELEASE 4, SPARC processor specific code
 *   this file is a part of xtrace project, see README for more details
 *
 * $Id: svr4-sparc.c,v 1.2 2002/04/21 08:11:31 drozd Exp $
 * $Log: svr4-sparc.c,v $
 * Revision 1.2  2002/04/21 08:11:31  drozd
 * autoconf support. svr4 is still broken
 *
 * Revision 1.1.1.1  2002/02/21 08:55:10  drozd
 * import
 *
 */

#include <stdio.h>
#include <procfs.h>

#include "xtrace.h"
#include "svr4.h"

static int isleaf(int, vaddr_t);


/*
 * Verify, is the current function leaf - optimized or not
 * normal function return:  ret  (jmpl %i7+8,%g0) (0x81c7e008)
 * leaf function return:    retl (jmpl %o7+8,%g0) (0x81c3e008)
 * Parameters:
 *   fd - file descriptor to the open address space file
 *   npc - `next instruction' program counter
 * Return:
 *   1 if a leaf procedure, 0 if not
 */
int
isleaf(int fd, vaddr_t npc)
{
#define NORMAL_RET	0x81c7e008
#define LEAF_RET	0x81c3e008

	unsigned scan_area[BUFSIZ];	/* beginning of function*/
	int i, found = 0;

	(void) lseek(fd, (off_t) npc, SEEK_SET);

	if ((read(fd, (void *) scan_area, sizeof(scan_area))) != sizeof(scan_area)) {
		(void) fprintf(stderr, "isleaf read()");
		return 0;
	}
	
	for (i = 0; i < (sizeof(scan_area)/sizeof(scan_area[0])); i++) {
		if (scan_area[i]  == NORMAL_RET) {
			found = 1;	/* not leaf*/
			break;
		} else if (scan_area[i]  == LEAF_RET) {
			found = 2;	/* leaf */
		}
	}
	
	if (!found)
		fprintf(stderr, "error scanning function\n");
	
	return (found == 2 ? 1 : 0);
}

/*
 * Get call data.
 * Parameters:
 *    op - pre to options structure
 *    sp - stack pointer
 *    rta - return address to be saved there
 *    prevsp - previous stack pointer value to be saved there
 *    args - ptr to null terminated array of arguments to be saved there
 * Return:
 *    0 on success, -1 on error
 */
int
xtr_get_call_data(const struct opt *op, struct calldata *cdp)
{
	static unsigned frame[0x10];
	struct svr4data *dp = (struct svr4data *) op->data;

	/* read frame from address space */
	(void) lseek(dp->afd, (off_t) cdp->fp, SEEK_SET);
	
	if (read(dp->afd, (void *) frame, sizeof(frame)) != sizeof(frame)) {
		perror("getframe read()"); return -1;
	}

	/*
	 * is it a leaf procedure ?
	 */
	/*
	if (isleaf(dp->afd, st.pr_reg[R_nPC]))
		args = (unsigned *) &st.pr_reg;
	else
		args = getargs(st.pr_reg[R_SP]);

	pc = st.pr_reg[R_PC];
	*/
	return -1;
}
