/*
 * Copyright (c) 2002 Toukan Ltd
 * All rights reserved
 *
 * Authors: Sergey Lyubka
 * Description:
 *   In this file syscalls are defined
 *   this file is a part of xtrace project
 *
 * $Id: syscall.c,v 1.11 2002/09/25 10:59:41 drozd Exp $
 */

#include <sys/syscall.h>

#include <stdio.h>

#include "xtrace.h"

static struct syscall __syscalls[] = {
	{ SYS_exit,	"exit", 1, { {INT, IN }, } },
	{ SYS_fork,	"fork", 0,  },
	{ SYS_read,	"read", 3, { {INT, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_write,	"write", 3, { {INT, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_open,	"open", 2, { {STRING, IN }, {HEX, IN }, } },
	{ SYS_close,	"close", 1, { {INT, IN }, } },
	{ SYS_wait4,	"wait4", 4, { {INT, IN }, {HEX, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_link,	"link", 2, { {STRING, IN }, {STRING, IN }, } },
	{ SYS_unlink,	"unlink", 1, { {STRING, IN }, } },
	
	{ SYS_chdir,	"chdir", 1, { {STRING, IN }, } },
	{ SYS_fchdir,	"fchdir", 1, { {INT, IN }, } },
	{ SYS_mknod,	"mknod", 3, { {STRING, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_chmod,	"chmod", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_chown,	"chown", 3, { {STRING, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_break,	"break", 1, { {HEX, IN }, } },			/* ??? */
	{ SYS_getpid,	"getpid", 0, },
	{ SYS_mount,	"mount", 4, { {STRING, IN }, {STRING, IN }, {INT, IN }, {HEX, IN }, } },
	
	{ SYS_setuid,	"setuid", 1, { {INT, IN }, } },
	{ SYS_getuid,	"getuid", 0 },
	{ SYS_geteuid,	"geteuid", 0 },
	{ SYS_ptrace,	"ptrace", 4, { {INT, IN }, {INT, IN }, {HEX, IN }, {INT, IN }} },
	{ SYS_access,	"access", 2, { {STRING, IN }, {INT, IN }, } },
	
	{ SYS_sync,	"sync", 0 },
	{ SYS_kill,	"kill", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_getppid,	"getppid", 0 },
	{ SYS_dup,	"dup", 1, { {INT, IN }, } },
	{ SYS_pipe,	"pipe", 1, { {HEX, IN }, } },
	{ SYS_getegid,	"getegid", 0 },
	{ SYS_profil,	"profil", 4, { {STRING, IN }, {INT, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_getgid,	"getgid", 0 },
	{ SYS_acct,	"acct", 1, { {STRING, IN }, } },
	{ SYS_sigaltstack,	"sigaltstack", 2, { {SIGALTSTACK, IN }, {SIGALTSTACK, IN }} },
	
	{ SYS_ioctl,	"ioctl", 3, { {INT, IN }, {INT, IN }, {IOCTL, IN }, } },
	{ SYS_reboot,	"reboot", 1, { {INT, IN }, } },
	{ SYS_symlink,	"symlink", 2, { {STRING, IN }, {STRING, IN }, } },
	{ SYS_readlink,	"readlink", 3, { {STRING, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_execve,	"execve", 3, { {STRING, IN }, {ARGV, IN }, {ARGV, IN }, } },
	{ SYS_umask,	"umask", 1, { {INT, IN }, } },
	{ SYS_chroot,	"chroot", 1, { {STRING, IN }, } },
	{ SYS_msync,	"msync", 3, { {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_vfork,	"vfork", 0 },
	{ SYS_munmap,	"munmap", 2, { {HEX, IN }, {INT, IN }, } },
	{ SYS_mprotect,	"mprotect", 3, { {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_madvise,	"madvise", 3, { {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_mincore,	"mincore", 3, { {HEX, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_getgroups,	"getgroups", 2, { {INT, IN }, {HEX, IN } } },
	{ SYS_setgroups,	"setgroups", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_getpgrp,	"getpgrp", 0 },
	{ SYS_setpgid,	"setpgid", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_setitimer,	"setitimer", 3, { {INT, IN }, {ITIMER, IN }, {ITIMER, IN }, } },
	{ SYS_swapon,	"swapon", 1, { {STRING, IN }, } },
	{ SYS_getitimer,	"getitimer", 2, { {INT, IN }, {ITIMER, IN }, } },
	{ SYS_dup2,	"dup2", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_fcntl,	"fcntl", 3, { {INT, IN }, {FCNTL, IN }, {HEX, IN }, } },
	
	{ SYS_select,	"select", 5,
	     { {INT, IN }, {FDSET, IN }, {FDSET, IN }, {FDSET, IN }, {TIMEVAL, IN }, } },
	{ SYS_fsync,	"fsync", 1, { {INT, IN }, } },
	{ SYS_setpriority,	"setpriority", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_getpriority,	"getpriority", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_gettimeofday,	"gettimeofday", 2, { {TIMEVAL, IN }, {TIMEZONE, IN }, } },
	{ SYS_getrusage,	"getrusage", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_readv,	"readv", 3, { {INT, IN }, {IOVEC, IN }, {INT, IN }, } },
	{ SYS_writev,	"writev", 3, { {INT, IN }, {IOVEC, IN }, {INT, IN }, } },
	{ SYS_settimeofday,	"settimeofday", 2, { {TIMEVAL, IN }, {TIMEZONE, IN }, } },
	{ SYS_fchown,	"fchown", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_fchmod,	"fchmod", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_setreuid,	"setreuid", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_setregid,	"setregid", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_rename,	"rename", 2, { {STRING, IN }, {STRING, IN }, } },
	{ SYS_flock,	"flock", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_mkdir,	"mkdir", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_rmdir,	"rmdir", 1, { {STRING, IN }, } },
	{ SYS_setsid,	"setsid", 0 },
	{ SYS_quotactl,	"quotactl", 4, { {STRING, IN }, {INT, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_statfs,	"statfs", 2, { {INT, IN }, {STATFS, IN }, } },
	{ SYS_fstatfs,	"fstatfs", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_uname,	"uname", 0 },
	{ SYS_pread,	"pread", 4, { {INT, IN }, {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_pwrite,	"pwrite", 4, { {INT, IN }, {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_setgid,	"setgid", 1, { {INT, IN }, } },
	{ SYS_stat,	"stat", 2, { {STRING, IN }, {HEX, IN }, } },
	{ SYS_fstat,	"fstat", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_lstat,	"lstat", 2, { {STRING, IN }, {HEX, IN }, } },
	{ SYS_getrlimit,	"getrlimit", 2, { {INT, IN }, {RLIMIT, IN }, } },
	{ SYS_setrlimit,	"setrlimit", 2, { {INT, IN }, {RLIMIT, IN }, } },
	{ SYS_mmap,	"mmap", 6, { {HEX, IN }, {INT, IN }, {HEX, IN }, {HEX, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_lseek,	"lseek", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_truncate,	"truncate", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_ftruncate,	"ftruncate", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_lchown,	"lchown", 3, { {STRING, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_getdents,	"getdents", 3, { {INT, IN }, {HEX, IN }, {INT, IN }, } },

	{ SYS_getsid,	"getsid", 1, { {INT, IN }, } },
	{ SYS_setresuid,	"setresuid", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_setresgid,	"setresgid", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_mlockall,	"mlockall", 1, { {HEX, IN }, } },		/* ??? */
	{ SYS_munlockall,	"munlockall", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_sched_setparam,	"sched_setparam", 2, { {INT, IN }, {SHEDPARAM, IN }, } },
	{ SYS_sched_getparam,	"sched_getparam", 2, { {INT, IN }, {SHEDPARAM, IN }, } },
	{ SYS_sched_setscheduler,	"sched_setscheduler", 3,
		{ {INT, IN }, {INT, IN }, {SHEDPARAM, IN }, } },
	{ SYS_sched_getscheduler,	"sched_getscheduler", 1, { {INT, IN }, } },
	{ SYS_sched_yield,	"sched_yield", 0 },
	{ SYS_sched_get_priority_max,	"sched_get_priority_max", 1, { {INT, IN }, } },
	{ SYS_sched_get_priority_min,	"sched_get_priority_min", 1, { {INT, IN }, } },
	{ SYS_sched_rr_get_interval,	"sched_rr_get_interval", 2,
		{ {INT, IN }, {TIMESPEC, IN }, } },
	{ SYS_sendfile,	"sendfile", 7, { {INT, IN }, {INT, IN }, {INT, IN }, {INT, IN }, {HEX, IN }, {HEX, IN }, {INT, IN }, } },

	{ SYS_sigprocmask,	"sigprocmask", 3, { {INT, IN }, {SIGSET, IN }, {SIGSET, IN }, } },
	{ SYS_sigsuspend,	"sigsuspend", 1, { {SIGSET, IN }, } },
	{ SYS_sigaction,	"sigaction", 3, { {INT, IN }, {SIGACTION, IN }, {SIGACTION, IN },} },
	{ SYS_sigpending,	"sigpending", 1, { {SIGSET, IN }, } },
	{ SYS_sigreturn,	"sigreturn", 1, { {HEX, IN }, } },
	{ SYS_getresuid,	"getresuid", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_getresgid,	"getresgid", 1, { {HEX, IN }, } },	/* ??? */

	{ SYS_nanosleep,	"nanosleep", 2, { {TIMESPEC, IN }, {TIMESPEC, OUT }, } },
	{ SYS_setdomainname,	"setdomainname", 2, { {STRING, IN }, {INT, IN } , } },
	{ SYS_mlock,	"mlock", 2, { {HEX, IN }, {INT, IN }, } },
	{ SYS_munlock,	"munlock", 2, { {HEX, IN }, {INT, IN }, } },
	{ SYS_getpgid,	"getpgid", 1, { {INT, IN }, } },
	{ SYS_poll,	"poll", 3, { {HEX, IN }, {INT, IN }, {INT, IN }, } },

	/* FREEBSD specific syscalls */

#ifdef FREEBSD
	{ SYS_ktrace,	"ktrace", 4, { {STRING, IN }, {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_getlogin,	"getlogin", 0 },
	{ SYS_setlogin,	"setlogin", 1, { {STRING, IN }, } },
	{ SYS_fchflags,	"fchflags", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_chflags,	"chflags", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_recvmsg,	"recvmsg", 3, { {INT, IN }, {MSGHDR, IN }, {INT, IN }, } },
	{ SYS_sendmsg,	"sendmsg", 3, { {INT, IN }, {MSGHDR, IN }, {INT, IN }, } },
	{ SYS_recvfrom,	"recvfrom", 6, { {INT, IN }, {HEX, IN }, {INT, IN }, {INT, IN }, {SOCKADDR, IN }, {HEX, IN }, } },
	{ SYS_accept,	"accept", 3, { {INT, IN }, {SOCKADDR, IN }, {HEX, IN }, } },
	{ SYS_getpeername,"getpeername", 3, { {INT, IN }, {SOCKADDR, IN }, {HEX, IN }, } },
	{ SYS_getsockname,"getsockname", 3, { {INT, IN }, {SOCKADDR, IN }, {HEX, IN }, } },
	{ SYS_unmount,	"unmount", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_getfsstat,	"getfsstat", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_syscall,	"syscall", 0,  },
	{ SYS_revoke,	"revoke", 1, { {STRING, IN }, } },
	{ SYS_socket,	"socket", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_connect,	"connect", 3, { {INT, IN }, {SOCKADDR, IN }, {INT, IN }, } },
	{ SYS_setegid,	"setegid", 1, { {INT, IN }, } },
	{ SYS_bind,	"bind", 3, { {INT, IN }, {SOCKADDR, IN }, {INT, IN }, } },
	{ SYS_setsockopt,"setsockopt", 5, { {INT, IN }, {INT, IN }, {INT, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_listen,	"listen", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_getsockopt,"getsockopt", 5, { {INT, IN }, {INT, IN }, {INT, IN }, {HEX, IN }, {HEX, IN }, } },
	{ SYS_nfssvc,	"nfssvc", 2, { {INT, IN }, {HEX, IN }, } },	
	{ SYS_getfh,	"getfh", 2, { {STRING, IN }, {HEX, IN }, } },
	{ SYS_ntp_adjtime,	"ntp_adjtime", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_seteuid,	"seteuid", 1, { {INT, IN }, } },
	{ SYS_pathconf,	"pathconf", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_getdirentries,"getdirentries", 4, { {INT, IN }, {HEX, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_fpathconf,	"fpathconf", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_sbrk,	"sbrk", 1, { {INT, IN }, } },
	{ SYS_sstk,	"sstk", 1, { {HEX, IN }, } },			/* ??? */
	{ SYS_vadvise,	"vadvise", 1, { {HEX, IN }, } },		/* ??? */
	{ SYS_getdtablesize,	"getdtablesize", 0 },
	{ SYS_sysarch,	"sysarch", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_semsys,	"semsys", 3, { {HEX, IN }, {HEX, IN }, {HEX, IN }, } },	/* ??? */
	{ SYS_msgsys,	"msgsys", 3, { {HEX, IN }, {HEX, IN }, {HEX, IN }, } },	/* ??? */
	{ SYS_shmsys,	"shmsys", 3, { {HEX, IN }, {HEX, IN }, {HEX, IN }, } },	/* ??? */
	{ SYS_mkfifo,	"mkfifo", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_sendto,"sendto", 6, { {INT, IN }, {HEX, IN }, {INT, IN }, {INT, IN }, {SOCKADDR, IN }, {INT, IN }, } },
	{ SYS_shutdown,	"shutdown", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_utimes,	"utimes", 2, { {STRING, IN }, {TIMEVAL, IN }, } },
	{ SYS_adjtime,	"adjtime", 2, { {TIMEVAL, IN }, {TIMEVAL, IN }, } },
	{ SYS_rtprio,	"rtprio", 3, { {INT, IN }, {INT, IN }, {RTPRIO, IN }, } },
	{ SYS_getdomainname,	"getdomainname", 2, { {STRING, IN }, {INT, IN },  } },
	{ SYS_socketpair,	"socketpair", 4, { {INT, IN }, {INT, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS___syscall,	"__syscall", 0 },
	{ SYS___sysctl,	"__sysctl", 6, { {HEX, IN }, {INT, IN }, {HEX, IN }, {HEX, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_undelete,	"undelete", 1, { {STRING, IN }, } },
	{ SYS_futimes,	"futimes", 2, { {INT, IN }, {TIMEVAL, IN }, } },
	{ SYS___semctl,	"__semctl", 4, { {INT, IN }, {INT, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_semget,	"semget", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_semop,	"semop", 3, { {INT, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_msgctl,	"msgctl", 3, { {INT, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_msgget,	"msgget", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS_msgsnd,	"msgsnd", 4, { {INT, IN }, {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_msgrcv,	"msgrcv", 5, { {INT, IN }, {HEX, IN }, {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_shmat,	"shmat", 3, { {INT, IN }, {HEX, IN }, {INT, IN }, } },
	{ SYS_shmctl,	"shmctl", 3, { {INT, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_shmdt,	"shmdt", 1, { {HEX, IN }, } },
	{ SYS_shmget,	"shmget", 3, { {INT, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_clock_gettime,	"clock_gettime", 2, { {INT, IN }, {TIMESPEC, IN }, } },
	{ SYS_clock_settime,	"clock_settime", 2, { {INT, IN }, {TIMESPEC, IN }, } },
	{ SYS_clock_getres,	"clock_getres", 2, { {INT, IN }, {TIMESPEC, IN }, } },
	{ SYS_minherit,	"minherit", 3, { {HEX, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS_rfork,	"rfork", 1, { {INT, IN }, } },
	{ SYS_openbsd_poll,	"openbsd_poll", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_issetugid,	"issetugid", 0 },
	{ SYS_lchmod,	"lchmod", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS_netbsd_lchown,	"netbsd_lchown", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_lutimes,	"lutimes", 2, { {STRING, IN }, {TIMEVAL, IN }, } },
	{ SYS_netbsd_msync,	"netbsd_msync", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_nstat,	"nstat", 1, { {HEX, IN }, } },			/* ??? */
	{ SYS_nfstat,	"nfstat", 1, { {HEX, IN }, } },		/* ??? */
	{ SYS_nlstat,	"nlstat", 1, { {HEX, IN }, } },		/* ??? */
	{ SYS_fhstatfs,	"fhstatfs", 2, { {INT, IN }, {STATFS, IN }, } },
	{ SYS_fhopen,	"fhopen", 2, { {HEX, IN }, {INT, IN }, } },
	{ SYS_fhstat,	"fhstat", 2, { {HEX, IN }, {STAT, IN }, } },
	{ SYS_modnext,	"modnext", 1, { {INT, IN }, } },
	{ SYS_modstat,	"modstat", 2, { {INT, IN }, { MODSTAT, IN }, } },
	{ SYS_modfnext,	"modfnext", 1, { {INT, IN }, } },
	{ SYS_modfind,	"modfind", 1, { {STRING, IN }, } },
	{ SYS_kldload,	"kldload", 1, { {STRING, IN }, } },
	{ SYS_kldunload,	"kldunload", 1, { {INT, IN }, } },
	{ SYS_kldfind,	"kldfind", 1, { {STRING, IN }, } },
	{ SYS_kldnext,	"kldnext", 1, { {INT, IN }, } },
	{ SYS_kldstat,	"kldstat", 2, { {INT, IN }, { KLDSTAT, IN }, } },
	{ SYS_kldfirstmod,	"kldfirstmod", 1, { {INT, IN }, } },
	{ SYS_aio_return,	"aio_return", 1, { {HEX, IN }, } },
	{ SYS_aio_suspend,	"aio_suspend", 3, { {HEX, IN }, {INT, IN }, {TIMESPEC, IN }, } },
	{ SYS_aio_cancel,	"aio_cancel", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_aio_error,	"aio_error", 1, { {HEX, IN }, } },
	{ SYS_aio_read,	"aio_read", 1, { {HEX, IN }, } },
	{ SYS_aio_write,	"aio_write", 1, { {HEX, IN }, } },
	{ SYS_lio_listio,	"lio_listio", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_yield,	"yield", 1, { {HEX, IN }, } },			/* ??? */
	{ SYS_thr_sleep,	"thr_sleep", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_thr_wakeup,	"thr_wakeup", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS___getcwd,	"__getcwd", 1, { {HEX, IN }, } },
	{ SYS_kldsym,	"kldsym", 3, { {INT, IN }, {INT, IN }, {HEX, IN }, } },
	{ SYS_jail,	"jail", 1, { {HEX, IN }, } },
	{ SYS___acl_get_file,	"__acl_get_file", 2, { {STRING, IN }, {INT, IN }, } },
	{ SYS___acl_set_file,	"__acl_set_file", 3, { {STRING, IN }, {INT, IN }, {INT, IN }, } },
	{ SYS___acl_get_fd,	"__acl_get_fd", 1, { {INT, IN }, } },
	{ SYS___acl_set_fd,	"__acl_set_fd", 2, { {INT, IN }, {INT, IN }, } },
	{ SYS___acl_delete_file,	"__acl_delete_file", 1, { {HEX, IN }, } },/* ??? */
	{ SYS___acl_delete_fd,	"__acl_delete_fd", 1, { {HEX, IN }, } },/* ??? */
	{ SYS___acl_aclcheck_file,	"__acl_aclcheck_file", 1, { {HEX, IN }, } },/* ??? */
	{ SYS___acl_aclcheck_fd,	"__acl_aclcheck_fd", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_extattrctl,	"extattrctl", 1, { {HEX, IN }, } },	/* ??? */
	{ SYS_extattr_set_file,	"extattr_set_file", 1, { {HEX, IN }, } },/* ??? */
	{ SYS_extattr_get_file,	"extattr_get_file", 1, { {HEX, IN }, } },/* ??? */
	{ SYS_extattr_delete_file,	"extattr_delete_file", 1, { {HEX, IN }, } }, /*??? */
	{ SYS_aio_waitcomplete,	"aio_waitcomplete", 2, { {HEX, IN }, {TIMESPEC, IN }, } },
	{ SYS_utrace,	"utrace", 1, { {HEX, IN }, } },		/* ??? */
	{ SYS_kqueue,	"kqueue", 0 },
	{ SYS_kevent,	"kevent", 6,
	    { {INT, IN }, {KEVENT, IN }, {INT, IN }, {KEVENT, IN }, {INT, IN }, {TIMESPEC, IN }, } },
#endif /* FREEBSD */


	/* LINUX specific syscalls */
#ifdef LINUX
	{ SYS__newselect, "select", 5,
	    { {INT, IN }, {FDSET, IN }, {FDSET, IN }, {FDSET, IN }, {TIMEVAL, IN }, } },
	{ SYS_lstat64, "lstat64", 2, { {STRING, IN }, {HEX, IN }, } },
	{ SYS_brk, "brk", 1, { {HEX, IN }, } },
	{ SYS_mmap2, "mmap2", 6, { {HEX, IN }, {INT, IN}, {HEX, IN},
	    {HEX,IN}, {INT, IN}, {HEX, IN}, } },
	{ SYS_fstat64, "fstat64", 2, { {INT, IN }, {HEX, IN }, } },
	{ SYS_rt_sigprocmask, "rt_sigprocmask", 3, { {INT, IN },
	    {SIGSET, IN }, {SIGSET, OUT}, } },
	{ SYS_rt_sigaction, "rt_sigaction", 3, { {INT, IN },
	    {SIGACTION, IN }, {SIGACTION, OUT}, } },
#endif /* LINUX */
};

/*
 * Return syscall's name by the given number
 * Parameters:
 *   no - syscall number
 * Return:
 *   syscall's name
 */
const struct syscall *
xtr_lookup_syscall(int no)
{
	static struct syscall unknown = { -1, NULL, 0};
	static char name[32];
	const struct syscall *retval = 0;
	int i;

	for (i = 0; i < sizeof(__syscalls)/sizeof(struct syscall); i++)
		if (__syscalls[i].no == no) {
			retval = &__syscalls[i];
			break;
		}

	if (retval == NULL) {
		(void) snprintf(name, sizeof(name), "unknown[no %d]", no);
		unknown.name = name;
		retval = &unknown;
	}

	return (retval);
}

